/*
 * 11/01/2002
 *
 * Utilities
 * Auteur: Vouriot Thierry (Yeri)
 * Email: yeri@fr.st, thierry.vouriot@free.fr
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife;

import java.io.*;
import java.awt.*;
import java.net.*;
import javax.swing.*;
import tvouriot.elife.*;

/** Class regroupant diffrentes fonctions utiles
 */
public class Utilities {
    
    /** Ouvrir une fentre <I>dialog</I> permettant  l'utilisateur d'entrer une donn
     * @param parent composant appelant cette fentre
     * @param message message  afficher dans la fentre
     * @param titre titre de la fentre
     * @return retourne la valeur entre par l'utilisateur
     */
    public static String showInputQuestion(Component parent, String message, String titre) {
        return JOptionPane.showInputDialog(parent,message,titre,JOptionPane.QUESTION_MESSAGE);
    }
    
    /** Affiche un message d'erreur
     * @param message message  afficher
     */
    public static void showError(String message) {
        JOptionPane.showMessageDialog(null, message, "Erreur", JOptionPane.ERROR_MESSAGE);
    }
    
    /** Affiche une fentre permettant de demander  l'utilisateur si il veut enregistrer un fichier
     * @param parent composant appelant cette fentre
     * @param name nom du fichier
     * @return entier correspondant aux boutons <I>yes no</I> et <I>cancel</I> (voir <CODE>JOptionPane</CODE>)
     */
    public static int showWantToSaveFile(Component parent, String name) {
        return JOptionPane.showConfirmDialog(parent,"Voulez-vous enregistrer le fichier "+name+" ?","Attention",JOptionPane.YES_NO_CANCEL_OPTION);
    }
    
    /** Affiche un message dans une fentre <I>dialog</I>
     * @param title titre
     * @param message message  afficher
     */
    public static void showMessage(String title, String message) {
        JOptionPane.showMessageDialog(null, message, title, JOptionPane.INFORMATION_MESSAGE);
    }
    
    /** Permet d'mettre un son "beep"
     */
    public static void beep() {
        Toolkit.getDefaultToolkit().beep();
    }
    
    /** Cration d'un objet <CODE>Image</CODE>  partir d'un fichier
     * @param picture chemin d'accs relatif de l'image
     * @param source class  partir de laquelle le chemin d'accs est recherch
     * @return l'image
     */
    public static Image getImage(String picture, Class source) {
        return Toolkit.getDefaultToolkit().getImage(source.getResource(picture));
    }
    
    /** Cration d'un objet <CODE>URL</CODE>  partir d'un fichier
     * @param path chemin d'accs relatif du fichier
     * @param source class  partir de laquelle le chemin d'accs est recherch
     * @return url du fichier
     */
    public static URL getURL(String path, Class source) {
        return source.getResource(path);
    }
    
    /** Cration d'un objet <CODE>ImageIcon</CODE>  partir d'un fichier
     * @param picture chemin d'accs relatif de l'image
     * @param source class  partir de laquelle le chemin d'accs est recherch
     * @return icne
     */
    public static ImageIcon getIcon(String picture, Class source) {
        return new ImageIcon(source.getResource(picture));
    }
    
    /** Renvoie les coordonnes pour que le second composant soit positionn au milieu du premier
     * @return coordonnes
     * @param comp1 premier composant
     * @param comp2 second composant
     */
    public static Point getCenterComponentChild(Component comp1, Component comp2) {
        return new Point(comp1.getX()+comp1.getWidth()/2-comp2.getWidth()/2,comp1.getY()+comp1.getHeight()/2-comp2.getHeight()/2);
    }
    
    /** Renvoie les coordonnes pour que le composant soit au centre de l'cran
     * @return coordonnes
     * @param comp composant  centrer
     */
    public static Point getCenterComponent(Component comp) {
        Dimension reso = comp.getToolkit().getScreenSize();
        return new Point(reso.width/2-comp.getWidth()/2,reso.height/2-comp.getHeight()/2);
    }
    
    /** Renvoie l'extension d'un fichier pass en paramtre
     * @param f fichier
     * @return extension du fichier
     */
    public final static String getFileExtension(File f) {
        String path = f.getName();
        int i = path.lastIndexOf('.');
        if (i<0)
            return "";
        return path.substring(i);
    }
    
    /** Charge une gometrie  partir des proprits de l'application et
     * l'applique  la fentre passe en paramtre
     * @param win fentre  modifier
     * @param name nom de la proprit contenant la gometrie
     */
    public static void loadGeometry(Window win, String name) {
        int x, y, width, height;
        
        try {
            width = Integer.parseInt(Elife.getProperty(name + ".width"));
            height = Integer.parseInt(Elife.getProperty(name + ".height"));
        } catch(NumberFormatException nf) {
            Dimension size = win.getSize();
            width = size.width;
            height = size.height;
        }
        
        try {
            x = Integer.parseInt(Elife.getProperty(name + ".x"));
            y = Integer.parseInt(Elife.getProperty(name + ".y"));
        } catch(NumberFormatException nf) {
            Dimension screen = win.getToolkit().getScreenSize();
            x = (screen.width - width) / 2;
            y = (screen.height - height) / 2;
        }
        win.pack();
        win.setLocation(x, y);
        win.setSize(width, height);
    }
    
    /** Retourne une Fonte en fonction d'une cl dans la table des proprits
     * @param key cl de la fonte correspondant au cls suivantes ("key.fontname","key.fontsize","key.fontstyle")
     * @return nouvelle fonte
     */
    public static Font getFont(String key) {
        int fontSize, fontStyle;
        String fontFamily;
        
        try {
            fontSize = Integer.parseInt(Elife.getProperty(key + ".fontsize"));
        } catch (Exception e) {
            fontSize = 12;
        }
        try {
            fontStyle = Integer.parseInt(Elife.getProperty(key + ".fontstyle"));
        } catch (Exception e) {
            fontStyle = 0;
        }
        fontFamily = Elife.getProperty(key + ".fontname");
        return new Font(fontFamily, fontStyle, fontSize);
    }
    
    /** Permet de changer le <I>look & feel</I> de l'interface
     * @param c composant dont il faut changer le look
     * @param plaf nom du <I>look & feel</I> ("metal", "motif", "windows", "macintosh")
     */
    public static void changePlaf(Component c, String plaf) {
        plaf = plaf.toLowerCase();
        JOptionPane DialogBox = new JOptionPane("Info",JOptionPane.ERROR_MESSAGE);
        try {
            
            if (plaf.equals("metal"))
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            if (plaf.equals("cde/motif"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            if (plaf.equals("windows"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            SwingUtilities.updateComponentTreeUI(c);
            
        }
        catch (UnsupportedLookAndFeelException e) {
            String s = "Look & Feel non disponible sur votre systme.\n";
            if (plaf.equals("windows"))
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf.equals("mac"))
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(c,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (IllegalAccessException e) {
            String s = "Look & Feel non accessible sur votre systme.\n";
            if (plaf.equals("windows"))
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf.equals("mac"))
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(c,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (ClassNotFoundException e) {
            String s = "Look & Feel non trouv.\n";
            if (plaf.equals("windows"))
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf.equals("mac"))
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(c,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (InstantiationException e) {
            System.out.println
            ("Ce Look & Feel ne peut tre instanci.");
        }
        catch (Exception e) {
            System.out.println("Erreur d'excution.");
        }
        DialogBox = null;
    }
}