/*
 * 11/01/2002
 *
 * DiaFindReplace
 * Auteur: Vouriot Thierry (Yeri)
 * Email: yeri@fr.st, thierry.vouriot@free.fr
 * Web: http://www.yeri.fr.st
 * (Classe reprise et modifier  partir de l'editeur Jext www.jext.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.gui;

import java.awt.*;
import gnu.regexp.*;
import javax.swing.*;
import java.awt.event.*;
import tvouriot.elife.*;

/** Fentre permettant d'effectuer des recherches et/ou des remplacement dans le texte
 */
public class DiaFindReplace extends JDialog implements ActionListener {
    
    /** Mode recherche uniquement */
    public static final int SEARCH = 1;
    /** Mode recherche et remplacement */
    public static final int REPLACE = 2;
    
    /** Type <CODE>SEARCH</CODE> ou <CODE>REPLACE</CODE> */
    private int type;
    /** Fentre principale de Elife */
    private ElifeFrame parent;
    /** Zone de texte o doit s'effectuer la recherche/remplacement */
    private ElifeTextArea textArea;
    /** Case  cocher permettant de choisir si la casse doit tre respecte */
    private JCheckBox checkIgnoreCase;
    /** Zone de texte o l'utilisateur doit rentrer le mot  rechercher */
    private JTextField fieldSearch = new JTextField(null, 15);
    /** Zone de texte o l'utilisateur doit rentrer le mot qui va remplacer le mot  rechercher */
    private JTextField fieldReplace = new JTextField(null, 15);
    /** Bouton "Rechercher" */    
    private ElifeButton btnFind;
    /** Bouton "Remplacer" */    
    private ElifeButton btnReplace;
    /** Bouton "Remplacer tout" */    
    private ElifeButton btnReplaceAll;
    /** Bouton "Annuler" */
    private ElifeButton btnCancel;
    
    /** Constructeur
     * @param parent Fentre principale de Elife
     * @param type type <CODE>SEARCH</CODE> ou <CODE>REPLACE</CODE>
     */
    public DiaFindReplace(ElifeFrame parent, int type) {
        
        super(parent, true);
        if (type == SEARCH)
            setTitle("Rechercher...");
        else if (type == REPLACE)
            setTitle("Remplacer...");
        setResizable(false);
        this.setSize(390,170);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        this.setLocation(Utilities.getCenterComponentChild(parent, this));
        
        this.parent = parent;
        this.textArea = parent.getTabbedPane().getCurrentPanelText().getTextArea();
        this.type = type;
        this.fieldSearch.setText(textArea.getFindPattern());
        this.fieldReplace.setText(textArea.getReplacePattern());
        
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());
        
        fieldSearch.setCursor(new Cursor(Cursor.TEXT_CURSOR));
        fieldReplace.setCursor(new Cursor(Cursor.TEXT_CURSOR));
        JPanel pnlMain = new JPanel(new GridLayout(5,1,2,3));
        JPanel pnlFind = new JPanel(new BorderLayout());
        JPanel pnlReplace = new JPanel(new BorderLayout());
        JPanel pnlBtnFind = new JPanel(new GridLayout(1,3));
        JPanel pnlBtnReplace = new JPanel(new GridLayout(1,3));
        
        JLabel findLabel = new JLabel("Rechercher :");
        JLabel replaceLabel = new JLabel("Remplacer :");
        
        btnFind = new ElifeButton("Rechercher",false);
        btnCancel = new ElifeButton("Annuler",false);
        btnReplace = new ElifeButton("Remplacer",false);
        btnReplaceAll = new ElifeButton("Remplacer tout",false);
        checkIgnoreCase = new JCheckBox("Ignorer la casse", true);
        
        if (type != REPLACE) fieldReplace.setEnabled(false);
        if (type != REPLACE) btnReplace.setEnabled(false);
        if (type != REPLACE) btnReplaceAll.setEnabled(false);
        
        pnlFind.add(findLabel,BorderLayout.WEST);
        pnlFind.add(fieldSearch,BorderLayout.CENTER);
        
        pnlBtnFind.add(new JPanel());
        pnlBtnFind.add(btnFind);
        pnlBtnFind.add(btnCancel);
        
        pnlReplace.add(replaceLabel,BorderLayout.WEST);
        pnlReplace.add(fieldReplace,BorderLayout.CENTER);
        
        pnlBtnReplace.add(new JPanel());
        pnlBtnReplace.add(btnReplace);
        pnlBtnReplace.add(btnReplaceAll);
        
        pnlMain.add(pnlFind);
        pnlMain.add(pnlBtnFind);
        pnlMain.add(pnlReplace);
        pnlMain.add(pnlBtnReplace);
        pnlMain.add(checkIgnoreCase);
        
        contentPane.add(pnlMain,BorderLayout.CENTER);
        contentPane.add(new JPanel(),BorderLayout.NORTH);
        
        btnFind.addActionListener(this);
        btnReplace.addActionListener(this);
        btnReplaceAll.addActionListener(this);
        btnCancel.addActionListener(this);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                exit();
            }
        });
        
        show();
    }
    
    /** Fermeture de la fentre et sauvegarde des mots  rechercher et  remplacer
     */    
    private void exit() {
        textArea.setFindPattern(fieldSearch.getText());
        textArea.setReplacePattern(fieldReplace.getText());
        dispose();
    }
    
    /** Fonction permettant la recherche du mot
     */    
    private void find() {
        if (!textArea.find(fieldSearch.getText(),
        textArea.getCaretPosition(), checkIgnoreCase.isSelected())) {
            int response = JOptionPane.showConfirmDialog(null, "Voulez-vous recommencer la recherche au dbut du fichier ?",
            " Elife ", JOptionPane.YES_NO_OPTION,
            JOptionPane.QUESTION_MESSAGE);
            switch (response) {
                case 0:
                    textArea.setCaretPosition(0);
                    find();
                    break;
                case 1:
                    break;
                default:
                    return;
            }
        }
    }
    
    /** Traitement suivant le bouton slctionn
     * @param evt Evnement grant l'action
     */
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();
        if (source == btnCancel)
            exit();
        else if (source == btnFind) {
            find();
        } else if (source == btnReplace) {
            try {
                if (!textArea.replaceAll(fieldSearch.getText(), fieldReplace.getText(),
                textArea.getSelectionStart(), textArea.getSelectionEnd(),
                checkIgnoreCase.isSelected()))
                    Utilities.beep();
                else
                    find();
            } catch (REException ree) { }
        } else if (source == btnReplaceAll) {
            try {
                if (!textArea.replaceAll(fieldSearch.getText(), fieldReplace.getText(), 0,
                textArea.getLength(), checkIgnoreCase.isSelected()))
                    Utilities.beep();
            } catch (REException ree) { }
        }
    }
}

