/*
 * 09/04/2002
 *
 * ElifeFrame
 * Auteur: Vouriot Thierry (Yeri)
 * Email: yeri@fr.st, thierry.vouriot@free.fr
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.gui;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import tvouriot.elife.menus.*;
import tvouriot.elife.toolbars.*;
import tvouriot.elife.actions.*;
import tvouriot.elife.images.*;
import tvouriot.elife.*;

/** Fentre principale de Elife
 */
public class ElifeFrame extends JFrame {
    
    /** Objet <CODE>JTabbedPane</CODE> contenant les zones de texte */
    private final FileTabbedPane tabbedPane;
    /** Panel contenant la barre de d'tats de Elife */
    private static PanelStatus pnlStatus;
    /** Barre d'outils principale */
    private static ElifeToolBar toolBar;
    /** Barre des menus principale */
    private static ElifeMenuBar menuBar;
    /** <CODE>JSplitPane</CODE> permettant de sparer l'explorateur de fichiers et les zones de texte */
    private JSplitPane splitVertical;
    /** Taille du sparateur du <CODE>JSplitPane</CODE> */
    private int dividerSize = 10;
    /** Objet <CODE>JTabbedPane</CODE> contenant les outils (situ dans le panneau gauche) */
    private ToolsTabbedPane tools;
    
    /** Constructeur
     */
    public ElifeFrame() {
        Elife.initUI();
        MenuAction.parent = this;
        Elife.initActions();
        Elife.initProperties();
        Utilities.changePlaf(this,Elife.getProperty("plaf"));
        
        
        tools = new ToolsTabbedPane(this);
        tabbedPane = new FileTabbedPane(this);
        splitVertical = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,tools,tabbedPane);
        splitVertical.setOneTouchExpandable(true);
        splitVertical.setDividerSize(dividerSize);
        
        toolBar = new ElifeToolBar();
        menuBar = new ElifeMenuBar();
        this.setJMenuBar(menuBar);
        this.setTitle("Elife");
        this.setIconImage(this.getToolkit().createImage(Utilities.getURL(Icons.ELIFE,Elife.class)));
        pnlStatus = new PanelStatus();
        Container contentPane = this.getContentPane();
        contentPane.add(toolBar,BorderLayout.NORTH);
        contentPane.add(pnlStatus,BorderLayout.SOUTH);
        contentPane.add(splitVertical,BorderLayout.CENTER);
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
                exit();
            }
        });
        init();
    }
    
    /** Initialise l'application en chargeant les paramtres du fichier de proprits */
    private void init() {
        tools.pnlFileExplorer.init();
        
        if (Elife.getProperty("toolbar.rollover").equals("off")) {
            toolBar.cmdNew.setRollover(false);
            toolBar.cmdOpen.setRollover(false);
            toolBar.cmdSave.setRollover(false);
            toolBar.cmdClose.setRollover(false);
            
            toolBar.cmdUndo.setRollover(false);
            toolBar.cmdRedo.setRollover(false);
            toolBar.cmdCut.setRollover(false);
            toolBar.cmdCopy.setRollover(false);
            toolBar.cmdPaste.setRollover(false);
            
            toolBar.cmdFind.setRollover(false);
            toolBar.cmdFindNext.setRollover(false);
            toolBar.cmdGoto.setRollover(false);
            toolBar.cmdReplace.setRollover(false);
            
            toolBar.cmdAbout.setRollover(false);
            toolBar.cmdDoc.setRollover(false);
        }
        
        if (Elife.getProperty("tools").equals("off")) {
            this.hideOrShowTools();
        }
        if (Elife.getProperty("toolbar").equals("off"))  {
            toolBar.setVisible(false);
            menuBar.mnuItemToolbar.setSelected(false);
        }
        if (Elife.getProperty("statusbar").equals("off")) {
            pnlStatus.setVisible(false);
            menuBar.mnuItemStatusbar.setSelected(false);
        }
        
        Utilities.loadGeometry(this,"Elife");
        
        if (Elife.getProperty("tools").equals("off")) splitVertical.setDividerSize(0);
    }
    
    /** Retourne le rpertoire affich par l'explorateur de fichiers
     * @return rpertoire de l'explorateur
     */
    public File getFileExplorerDirectory() {
        String path = tools.pnlFileExplorer.getCurrentDirectory();
        return path.equals(" ") ? null : new File(path);
    }
    
    /** Retourne le <CODE>JTabbedPane</CODE> contenant les zones de texte
     * @return zones de texte
     */
    public FileTabbedPane getTabbedPane() {
        return this.tabbedPane;
    }
    
    /** Barre d'outils principale de Elife
     * @return barre d'outils
     */
    public static ElifeToolBar getElifeToolBar() {
        return toolBar;
    }
    
    /** Barre de menus principale de Elife
     * @return barre de menus
     */
    public static ElifeMenuBar getElifeMenuBar() {
        return menuBar;
    }
    
    /** Retourne le panel contenant la barre d'tats de Elife
     * @return barre d'tats
     */
    public static PanelStatus getPanelStatus() {
        return pnlStatus;
    }
    
    /** Affiche ou cache l'explorateur de fichier
     */
    public void hideOrShowTools() {
        if (splitVertical.getLeftComponent() == null) {
            splitVertical.setLeftComponent(tools);
            Elife.setProperty("tools", "on");
            splitVertical.resetToPreferredSizes();
            splitVertical.setDividerSize(dividerSize);
            pnlStatus.setFileText(tools.pnlFileExplorer.getCurrentDirectory());
            menuBar.mnuItemTools.setSelected(true);
        }
        else {
            splitVertical.remove(tools);
            splitVertical.setDividerSize(0);
            Elife.setProperty("tools", "off");
            pnlStatus.setFileText("");
            menuBar.mnuItemTools.setSelected(false);
        }
    }
    
    /** Affiche ou cache la barre d'outils
     */
    public void hideOrShowToolbar() {
        if (toolBar.isVisible()) {
            toolBar.setVisible(false);
            Elife.setProperty("toolbar", "off");
        }
        else {
            toolBar.setVisible(true);
            Elife.setProperty("toolbar", "on");
        }
    }
    
    /** Affiche ou cache la barre de status
     */
    public void hideOrShowStatusbar() {
        if (pnlStatus.isVisible()) {
            pnlStatus.setVisible(false);
            Elife.setProperty("statusbar", "off");
        }
        else {
            pnlStatus.setVisible(true);
            Elife.setProperty("statusbar", "on");
        }
    }
    
    /** Appel des mthodes pour la fermeture de tous les fichiers et fermeture de l'application
     */
    public void closeAllandExit() {
        tabbedPane.supprAllTabs();
        if (tabbedPane.getTabCount()==1 && tabbedPane.firstTabIsEmptyAndNew())
            System.exit(0);
    }
    
    /** Appel des mthodes pour quitter l'application
     */
    public void exit() {
        Elife.setProperty("explorer.dir",tools.pnlFileExplorer.getCurrentDirectory());
        Elife.setProperty("explorer.hiddenfiles",tools.pnlFileExplorer.isShowHiddenFiles());
        Elife.setProperty("explorer.sorttype",tools.pnlFileExplorer.getSortType()+"");
        
        if (getX()>-5) Elife.setProperty("Elife.x",""+ getX());
        else Elife.setProperty("Elife.x",""+ 0);
        if (getY()>-5) Elife.setProperty("Elife.y",""+ getY());
        else Elife.setProperty("Elife.y",""+ 0);
        Elife.setProperty("Elife.height",""+this.getHeight());
        Elife.setProperty("Elife.width",""+this.getWidth());
        Elife.saveProps();
        this.closeAllandExit();
    }
}
