/*
 * 11/01/2002
 *
 * PanelFileExplorer
 * Auteur: Vouriot Thierry (Yeri)
 * Email: yeri@fr.st, thierry.vouriot@free.fr
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.gui;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import tvouriot.beans.filelistexplorer.*;
import tvouriot.elife.images.Icons;
import tvouriot.elife.*;

/** Panel contenant l'explorateur de fichiers
 */
public final class PanelFileExplorer extends JPanel {
    
    /** Explorateur de fichier */
    private static final FileListExplorer fle = new FileListExplorer();
    /** Barre d'outils de l'explorateur */
    private final ExplorerToolBar2 tb2 = new ExplorerToolBar2();
    
    /** Constructeur
     * @param frm Fentre principale de l'application
     */
    public PanelFileExplorer(final ElifeFrame frm) {
        super(new BorderLayout());
        
        HashMap hm = new HashMap();
        hm.put(".C",Utilities.getURL(Icons.C, Elife.class));
        hm.put(".H",Utilities.getURL(Icons.H, Elife.class));
        hm.put(".CPP",Utilities.getURL(Icons.CPP, Elife.class));
        hm.put(".CC",Utilities.getURL(Icons.CPP, Elife.class));
        hm.put(".JAVA",Utilities.getURL(Icons.JAVA, Elife.class));
        hm.put(".SPEC",Utilities.getURL(Icons.SPEC, Elife.class));
        hm.put(".HTM",Utilities.getURL(Icons.HTML, Elife.class));
        hm.put(".HTML",Utilities.getURL(Icons.HTML, Elife.class));
        hm.put(".XML",Utilities.getURL(Icons.XML, Elife.class));
        hm.put(".SQL",Utilities.getURL(Icons.SQL, Elife.class));
        
        fle.setSpecialIcons(hm);
        hm = null;
        fle.setBackground(Color.white);
        fle.setSelectionBackground(new Color(205,205,255));
        fle.setFixedCellWidth(120);
        fle.setSelectionBorderColor(new Color(150,150,200));
  
        this.add(new JScrollPane(fle),BorderLayout.CENTER);
        this.add(new ExplorerToolBar(),BorderLayout.NORTH);
        this.add(tb2,BorderLayout.SOUTH);
        
        fle.addFileListExplorerListener(new FileListExplorerListener() {
            public void fileSelected(FileListExplorerEvent evt) { }
            
            public void fileChoosen(FileListExplorerEvent evt) {
                frm.getTabbedPane().addTab(evt.getSelectedFile());
            }
            public void directoryChanged(FileListExplorerEvent evt) {
                frm.getPanelStatus().setFileText(getCurrentDirectory());
                tb2.lblNbFiles.setText(fle.getNbFiles()+ " fichier(s)   " + fle.getNbDirectories()+ " dossier(s)");
            }
        });
    }
    
    /** Initialisation de certains paramtres de l'explorateur */
    public void init() {
        this.setCurrentDirectory(Elife.getProperty("explorer.dir"));
        if (Elife.getProperty("explorer.hiddenfiles").equals("off"))
            tb2.chkHidden.setSelected(false);
        
        tb2.cmbSort.setSelectedIndex(Integer.parseInt(Elife.getProperty("explorer.sorttype")));
    }
    
    /** Teste si l'explorateur affiche ou non les fichiers cachs
     * @return "on" ou "off"
     */
    public String isShowHiddenFiles() {
        return fle.isShowHiddenFiles() ? "on" : "off";
    }
    
    /** Retourne le type de tri des fichiers de l'explorateur
     * @return type de tri (voir <CODE>tvouriot.beans.filelistexplorer.FileListExplorer</CODE>)
     */
    public int getSortType() {
        return fle.getSortType();
    }
    
    /** Retourne le rpertoire courant de l'explorateur de fichiers
     * @return rpertoire courant de l'explorateur de fichiers
     */
    public static String getCurrentDirectory() {
        return fle.getCurrentDirectory()!= null ? fle.getCurrentDirectory().getPath() : " ";
    }
    
    /** Modifie le rpertoire courant de l'explorateur de fichiers
     * @param path chemin d'accs du nouveau rpertoire courant
     */
    public static void setCurrentDirectory(String path) {
        if (path != null)
            fle.setCurrentDirectory(new File(path));
    }
    
    /** Barre d'outils (haut) de l'explorateur de fichiers
     */
    private static final class ExplorerToolBar extends JPanel {
        /** Bouton "Rpertoire parent" */
        private JButton cmdParent = new ElifeButton(Utilities.getIcon(Icons.PARENT, Elife.class),"Rpertoire parent",false);
        /** Bouton "Rpertoire suivant" */
        private JButton cmdNext = new ElifeButton(Utilities.getIcon(Icons.REPNEXT, Elife.class),"Rpertoire suivant",false);
        /** Bouton "Supprimer" */
        private JButton cmdDel = new ElifeButton(Utilities.getIcon(Icons.DEL, Elife.class),"Supprimer",false);
        /** Bouton "Crer un nouveau dossier" */
        private JButton cmdMkdir = new ElifeButton(Utilities.getIcon(Icons.MKDIR, Elife.class),"Crer un nouveau dossier",false);
        /** Bouton "Rafrachier" */
        private JButton cmdRefresh = new ElifeButton(Utilities.getIcon(Icons.REFRESH, Elife.class),"Rafrachir",false);
        /** Zone de texte pour le nouveau dossier */
        private JTextField txtDir = new JTextField();
        /** Bouton "Disques" */
        private JButton cmdRoots = new ElifeButton(Utilities.getIcon(Icons.ROOTS, Elife.class),"Disque(s)",false);
        /** Bouton "Rpertoire personnel" */
        private JButton cmdHome = new ElifeButton(Utilities.getIcon(Icons.HOME, Elife.class),"Rpertoire personnel",false);
        /** Dimension du sparateur de boutons */
        private final Dimension dimSeparator = new Dimension(2,2);
        private final Dimension dimSeparator2 = new Dimension(4,2);
        
        /** Constructeur
         */
        protected ExplorerToolBar() {
            
            this.setLayout(new BorderLayout());
            JToolBar tbC = new JToolBar();
            JToolBar tbS = new JToolBar();
            tbC.setFloatable(false);
            tbS.setFloatable(false);
            
            tbS.add(cmdParent);
            tbS.addSeparator(dimSeparator2);
            tbS.add(cmdNext);
            tbS.addSeparator(dimSeparator2);
            tbS.add(cmdDel);
            tbS.addSeparator(dimSeparator2);
            tbS.add(cmdRefresh);
            tbS.addSeparator(dimSeparator2);
            tbS.add(cmdRoots);
            tbS.addSeparator(dimSeparator2);
            tbS.add(cmdHome);
            tbC.add(cmdMkdir);
            tbC.addSeparator(dimSeparator);
            tbC.add(txtDir);
            
            this.add(tbC,BorderLayout.CENTER);
            this.add(tbS,BorderLayout.SOUTH);
            
            cmdParent.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    if (fle.getCurrentDirectory() != null)
                        fle.setCurrentDirectory(fle.getCurrentDirectory().getParentFile());
                }
            });
            cmdNext.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    File f = fle.getSelectedFile();
                    if (f != null && f.isDirectory())
                        fle.setCurrentDirectory(f);
                }
            });
            cmdDel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    File f = fle.getSelectedFile();
                    if (f != null) {
                        String s;
                        if (f.isDirectory()) {
                            s = "Voulez-vous vraiment supprimer le rpertoire " + f.getName() + " ?";
                            s += "\nAttention le rpertoire doit tre vide !";
                        }
                        else s = "Voulez-vous vraiment supprimer le fichier " + f.getName() + " ?";
                        if (JOptionPane.showConfirmDialog(null,s,"Suppresion",JOptionPane.YES_NO_OPTION) == JOptionPane.OK_OPTION)
                            fle.delete();
                    }
                }
            });
            cmdMkdir.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    String s = txtDir.getText().trim();
                    if (!s.equals("")) {
                        fle.mkDir(s);
                        txtDir.setText("");
                    }
                }
            });
            cmdRefresh.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    fle.setCurrentDirectory(fle.getCurrentDirectory());
                }
            });
            cmdRoots.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    fle.setCurrentDirectory(null);
                }
            });
            cmdHome.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    fle.setCurrentDirectory(Elife.HOME_DIRECTORY);
                }
            });
        }
    }
    
    /** Barre d'outils (bas) de l'explorateur de fichiers
     */
    private static final class ExplorerToolBar2 extends JPanel {   
        /** Label affichant lengthnombre de fichiers et de dossiers */
        protected JLabel lblNbFiles = new JLabel(" ");
        /** Bouton  ccher pour l'affichage/cachage des fichiers cachs */
        protected JCheckBox chkHidden = new JCheckBox("Afficher fichiers cachs",true);
        /** ComboBox pour le choix du type de tri */
        protected JComboBox cmbSort = new JComboBox();
        
        /** Constructeur
         */
        protected ExplorerToolBar2() {
            
            cmbSort.addItem("Trier par nom");
            cmbSort.addItem("Trier par extension");
            
            this.setLayout(new BorderLayout());
            JToolBar tbC = new JToolBar();
            JToolBar tbS = new JToolBar();
            JToolBar tbN = new JToolBar();
            tbC.setFloatable(false);
            tbS.setFloatable(false);
            tbN.setFloatable(false);
            
            tbC.add(chkHidden);
            tbN.add(cmbSort);
            tbS.add(lblNbFiles);
            
            this.add(tbC,BorderLayout.CENTER);
            this.add(tbS,BorderLayout.SOUTH);
            this.add(tbN,BorderLayout.NORTH);
            
            chkHidden.addItemListener(new ItemListener() {
                public void itemStateChanged(java.awt.event.ItemEvent itemEvent) {
                    if (chkHidden.isSelected())
                        fle.setShowHiddenFiles(true);
                    else fle.setShowHiddenFiles(false);
                    fle.refresh();
                }
            });
            cmbSort.addItemListener(new ItemListener() {
                public void itemStateChanged(java.awt.event.ItemEvent itemEvent) {
                    fle.setSortType(cmbSort.getSelectedIndex());
                    fle.refresh();
                }
            });
        }
    }
}