/*
 * 09/04/2002
 *
 * SplashScreen
 * Auteur: Vouriot Thierry (Yeri)
 * Email: yeri@fr.st, thierry.vouriot@free.fr
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.gui;

import java.awt.*;
import java.net.*;

/** Classe qui permet d'afficher une image d'introduction (sans cadre) au milieu de l'cran pendant un certain laps de temps.
 */
public class SplashScreen extends Window implements Runnable {
    
    /** Image  afficher */
    private Image image;
    /** Boolen permettant de savoir si l'affichage doit se terminer */
    private boolean terminated = false;
    
    /** Constructeur unique
     * @param img URL de l'image  afficher
     */
    public SplashScreen(URL img) {
        super(new Frame());
        
        // Load image and wait to complete
        image = Toolkit.getDefaultToolkit().getImage(img);
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image, 0);
        try {
            tracker.waitForID(0);
        }
        catch (Exception e) {}
        
        // Set window size and position
        int w = image.getWidth(this);
        int h = image.getHeight(this);
        setSize(w, h);
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screen.width - w)/2, (screen.height - h)/2);
        show();
        new Thread(this).start();
    }
    
    /** Termine l'affichage de l'image
     */
    public synchronized void setTerminated() {
        terminated = true;
        setVisible(false);
        notify();
    }
    
    /** Affiche l'image  l'cran
     * @param g Contexte graphique
     */
    public synchronized void paint(Graphics g) {
        g.drawImage(image, 0, 0, this);
        notify();
    }
    
    /** Affiche l'image tant que setTerminated n'est pas appel */
    public synchronized void run() {
        while (!terminated) {
            try { wait(); }
            catch (InterruptedException e) {}
        }
    }
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {
        image = null;
        super.finalize();
    }
}
