/*
 * 21/06/2002
 *
 * AbstractPanelOptions
 * Auteur: Vouriot Thierry (Yeri)
 * Email: yeri@fr.st, thierry.vouriot@free.fr
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.options;

import java.awt.*;
import javax.swing.*;
import tvouriot.elife.gui.*;

/** Class abstraite reprsentant les panneaux d'options
 */
public abstract class AbstractPanelOptions extends JPanel {
    
    private int y = 0;
    /** Gestionnaire de mise en page */
    protected GridBagLayout gridBag;
    /** Fentre principale de Elife */
    protected ElifeFrame parent;
    
    /** Constructeur
     *  @param fr fentre principale de Elife
     */
    public AbstractPanelOptions(ElifeFrame fr) {
        parent = fr;
        setLayout(gridBag = new GridBagLayout());
    }
    
    /**
     * Adds a labeled component in the pane. All the components
     * are placed on bottom of each other (vertically sorted).
     * @param label The label to be displayed next to the component
     * @param horizontalAlign label horizontal alignement
     * @param comp The component to be added
     */
    protected void addComponent(String label, int horizontalAlign, Component comp) {
        GridBagConstraints cons = new GridBagConstraints();
        cons.gridy = y++;
        cons.gridheight = 1;
        cons.gridwidth = 3;
        cons.fill = GridBagConstraints.BOTH;
        cons.weightx = 1.0f;
        
        cons.gridx = 0;
        cons.anchor = GridBagConstraints.WEST;
        JLabel l = new JLabel(label, horizontalAlign);
        gridBag.setConstraints(l, cons);
        add(l);
        
        cons.gridx = 3;
        cons.gridwidth = 1;
        cons.anchor = GridBagConstraints.EAST;
        
        gridBag.setConstraints(comp, cons);
        add(comp);
    }
    
    /**
     * Does the same as <code>addComponent(String, Component)</code>
     * but don't add a label next to the component.
     * @param comp The component to be added
     */
    protected void addComponent(Component comp) {
        GridBagConstraints cons = new GridBagConstraints();
        cons.gridy = y++;
        cons.gridheight = 1;
        cons.gridwidth = cons.REMAINDER;
        cons.fill = GridBagConstraints.NONE;
        cons.anchor = GridBagConstraints.WEST;
        cons.weightx = 1.0f;
        
        gridBag.setConstraints(comp, cons);
        add(comp);
    }
    
    /** Initilisation en fonction des proprits
     */
    protected abstract void init();
    
    /** Sauvegarde des nouvelles proprits
     */
    protected abstract void save();
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {
        gridBag = null;
        parent = null;
        super.finalize();
    }
}


