/*
 * 21/06/2002
 *
 * PanelOptionsEditor
 * Auteur: Vouriot Thierry (Yeri)
 * Email: yeri@fr.st, thierry.vouriot@free.fr
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.options;

import java.awt.*;
import javax.swing.*;
import tvouriot.elife.*;
import tvouriot.elife.gui.*;

/** Panneau contenant les options de l'diteur
 */
public final class PanelOptionsEditor extends AbstractPanelOptions {
    
    /** Case  cocher permettant de rendre le curseur carr */
    private JCheckBox chkBlockCaret = new JCheckBox("Curseur carr",false);
    /** Case  cocher permettant d'activer le surlignage des lignes */
    private JCheckBox chkLineHighlight = new JCheckBox("Surlignage des lignes",false);
    /** Zone de texte permettant de dfinir la largeur de la bordure */
    private IntTextField txtGutterWidth = new IntTextField();
    /** Zone de texte permettant de dfinir la largeur de la sparation entre la 
     *  bordure et l'diteur */
    private IntTextField txtBorderWidth = new IntTextField();
    /** Bouton permettant de choisir la police de l'diteur */
    private FontSelector cmdEditorFont = new FontSelector("editor");
    
    /** Constructeur
     *  @param fr fentre principale de Elife
     */
    public PanelOptionsEditor(ElifeFrame fr) {
        
        super(fr);
  
        this.addComponent("Police :",SwingUtilities.LEFT,cmdEditorFont);
        this.addComponent(chkBlockCaret);
        this.addComponent(chkLineHighlight);   
        this.addComponent("Bordure :",SwingUtilities.LEFT,txtGutterWidth);
        this.addComponent("Sparateur :",SwingUtilities.LEFT,txtBorderWidth);
        
        init();
    }
    
    /** Initilisation en fonction des proprits
     */
    protected void init() {
        chkBlockCaret.setSelected("on".equals(Elife.getProperty("editor.blockcaret")));
        chkLineHighlight.setSelected("on".equals(Elife.getProperty("editor.linehighlight")));
        txtGutterWidth.setText(Elife.getProperty("gutter.width"));
        txtBorderWidth.setText(Elife.getProperty("gutter.borderwidth"));
    }
    
    
    /** Sauvegarde des nouvelles proprits
     */
    protected void save() {
        Elife.setProperty("editor.blockcaret", chkBlockCaret.isSelected() ? "on" : "off");
        Elife.setProperty("editor.linehighlight", chkLineHighlight.isSelected() ? "on" : "off");
        if (txtGutterWidth.isValid() && txtGutterWidth.getValue()>=0)
            Elife.setProperty("gutter.width",txtGutterWidth.getText());
        if (txtBorderWidth.isValid() && txtBorderWidth.getValue()>=0)
            Elife.setProperty("gutter.borderwidth",txtBorderWidth.getText());
        cmdEditorFont.save();
        parent.getTabbedPane().refreshTextAreasProperties();
    }
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {        
        txtBorderWidth = null;
        txtGutterWidth = null;
        cmdEditorFont = null;
        chkBlockCaret = null;
        chkLineHighlight = null;
        super.finalize();
    }
}


