/*
 * 11/04/2002
 *
 * StatusToolBar
 * Auteur: Vouriot Thierry (Yeri)
 * Email: yeri@fr.st, thierry.vouriot@free.fr
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.toolbars;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import tvouriot.elife.*;
import tvouriot.elife.gui.*;
import tvouriot.elife.images.*;

/** Barre de status de l'diteur de texte */
public class StatusToolBar extends JToolBar {
    
    /** Label contenant les numros de lignes et de colonnes courantes */
    private JLabel lblLineCol = new JLabel();
    /** Bouton permettant d'activer/dsactiver le mode auto-indentation */
    private JToggleButton cmdAutoIndent = new JToggleButton(Utilities.getIcon(Icons.AUTOINDENT, Elife.class),true);
    /** Bouton permettant d'afficher/cacher la numrotation des lignes */
    private JButton cmdLineNumber = new JButton(Utilities.getIcon(Icons.LINENUM, Elife.class));
    /** Bouton permettant d'afficher/cacher les marqueurs de fin de lignes */
    private JToggleButton cmdEolMarker = new JToggleButton(Utilities.getIcon(Icons.EOLMARKERS, Elife.class));
    
    /** Constructeur */
    public StatusToolBar() {
        cmdAutoIndent.setToolTipText("Auto-indentation (on/off)");
        cmdAutoIndent.setMargin(new Insets(0,0,0,0));
        
        cmdLineNumber.setToolTipText("Numrotation des lignes (on/off)");
        cmdLineNumber.setMargin(new Insets(0,0,0,0));
        
        cmdEolMarker.setToolTipText("Marqueurs de fin de lignes (on/off)");
        cmdEolMarker.setMargin(new Insets(0,0,0,0));
        
        JPanel pnlButtons = new JPanel(new FlowLayout(FlowLayout.RIGHT,2,0));
        pnlButtons.add(cmdLineNumber);
        pnlButtons.add(cmdAutoIndent);
        pnlButtons.add(cmdEolMarker);
        
        this.setFloatable(false);
        this.add(lblLineCol);
        this.add(pnlButtons);
        
        cmdEolMarker.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                ((PanelTextEditor)getParent()).getTextArea().changeEOLMarkersPainted();
            }
        });
        
        cmdLineNumber.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                ((PanelTextEditor)getParent()).getTextArea().changeGutterPainted();
            }
        });
        
        cmdAutoIndent.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                ((PanelTextEditor)getParent()).getTextArea().changeAutoIndentMode();
            }
        });
    }
    
    /** Change le texte du label contenant les numros de lignes et de colonnes courantes */
    public void setLineCol(String s) {
        lblLineCol.setText(s);
    }
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {
        cmdEolMarker = null;
        cmdLineNumber = null;
        cmdAutoIndent = null;
        lblLineCol = null;
        super.finalize();
    }
}


